@@ -31,7 +31,7 @@ def getIndexSymbols(dateTime = 20100118,indexID = "000300",):
                     "indexID":indexID, "dateTime":int(dateTime)})
                 response = stub.getSymbols(indexData_pb2.JSONRequest(sReq=sReq))
                 jsonRsp = MessageToJson(response)
-                jsonRsp = json.loads(jsonRsp)
+                jsonRsp = json._loads(jsonRsp)
                 return jsonRsp["symbol"]
         except Exception as e:
             print(e)
@@ -40,9 +40,15 @@ def getIndexSymbols(dateTime = 20100118,indexID = "000300",):
 class IndexData:
     # todo:akshare 数据不全
     def __init__(self):
-        self.indexDfs = {}
+        self._indexDfs = {}
+        self._date_datas = {}
+        self._pointer = 0
 
-    def load(self,indexID = "000300"):
+    def create(self, begin_date=19890101, indexSymbol="SH.000300"):
+        self._date_datas = self.acquire_date_datas(indexSymbol)
+        self.update(begin_date)
+        
+    def _load(self, indexID = "000300"):
         _realPathFile  = (os.path.realpath(__file__))
         _path = os.path.split(_realPathFile)[0]
         fileName = os.sep.join([_path,r"data", "IndexCons", f"{indexID}指数成分股数据.xlsx"])  #
@@ -53,9 +59,9 @@ class IndexData:
         return df
 
     def getSymbols(self, dateTime:int, indexSymbol="SH.000300"):
-        if indexSymbol not in self.indexDfs:
-            self.indexDfs[indexSymbol] = self.load(indexID=indexSymbol[-6:])
-        indexDF = self.indexDfs[indexSymbol]
+        if indexSymbol not in self._indexDfs:
+            self._indexDfs[indexSymbol] = self._load(indexID=indexSymbol[-6:])
+        indexDF = self._indexDfs[indexSymbol]
         df = indexDF.iloc[indexDF.index <= str(dateTime)]
         symbols = []
         for index, row in df.iterrows():
@@ -70,7 +76,7 @@ class IndexData:
         symbols.sort()
         return symbols
     
-    # @Decorator.loadData(path="data")
+    # @Decorator._loadData(path="data")
     # def qryIndexStockCons(self,symbol = "SH.000300",**kwargs):
     #     import akshare as ak
     #     symbol = symbol[:2].lower()+symbol[-6:]
@@ -92,24 +98,23 @@ class IndexData:
     #         ret.append(symbol)
     #     ret.sort()
     #     print(ret)
-    
-    
+        
     def acquire_date_datas(self, indexSymbol="SH.000300"):
         """构造以日期为 key 的字典"""
-        if indexSymbol not in self.indexDfs:
-            self.indexDfs[indexSymbol] = self.load(indexID=indexSymbol[-6:])
+        if indexSymbol not in self._indexDfs:
+            self._indexDfs[indexSymbol] = self._load(indexID=indexSymbol[-6:])
         
-        datas     = {}
+        datas     = []
         symbols   = []
         last_date = 0
-        indexDF   = self.indexDfs[indexSymbol]        
+        indexDF   = self._indexDfs[indexSymbol]        
         for index, row in indexDF.iterrows():
             date = index.year * 10000 + index.month * 100 + index.day
             
             if date != last_date or last_date == 0:
-                datas[last_date] = symbols
-                if len(symbols) > 0: symbols = copy.copy(symbols)
                 last_date = date
+                datas.append([last_date, symbols])
+                if len(symbols) > 0: symbols = copy.copy(symbols)
                             
             symbol   = row["代码"][-2:] + "." + row["代码"][:6]            
             addOrDel = row[r"纳入/剔除"]
@@ -117,9 +122,51 @@ class IndexData:
             if    addOrDel == "纳入": symbols.append(symbol)
             elif  addOrDel == "剔除": symbols.remove(symbol)
             else: print(f"纳入/剔除 列的值错误 #{addOrDel}#")  
-        datas[last_date] = symbols              
+        
         return datas
     
+    def update(self, current_date):
+        assert(self._pointer >= 0 and self._pointer < len(self._date_datas))
+        
+        # 已经是最后的索引了
+        if self._pointer == len(self._date_datas) -1:
+            return
+    
+        idx = self._pointer
+        for idx in range(len(self._date_datas)):
+            # 如果已经到达最后一个索引
+            if idx == len(self._date_datas)-1:
+                self._pointer = idx
+                return
+            
+            # 如果刚好在正确区间，则更新并退出
+            if current_date >= self._date_datas[idx][0]  \
+                and current_date < self._date_datas[idx + 1][0]:
+                self._pointer = idx
+                return
+            
+            # 如果日期在区间前，则不处理
+            if current_date < self._date_datas[self._pointer][0]:
+                return
+                        
+    def acquire_datas_by_date(self, current_date):
+        """获取当前日期的集合"""
+        self.update(current_date)
+        
+        # 如果日期在所有区间前面，则返回空集
+        if self._pointer == 0 and current_date < self._date_datas[self._pointer][0]:
+            return []
+        
+        return self._date_datas[self._pointer][1]
+        
+    def acquire_all_codes(self):
+        """获取历史上所有的id"""
+        allcodes = []
+        for date,vs in self.acquire_date_datas():            
+            allcodes.extend(vs)
+        allcodes = list(set(allcodes))
+        return allcodes        
+    
 if __name__ == '__main__':
     # logging.basicConfig()
     # symbols500 = getIndexSymbols()
@@ -133,6 +180,6 @@ if __name__ == '__main__':
     r = idxData.getSymbols(dateTime = 20211222)
     print(len(r))
     
-    r = idxData.acquire_date_datas()
-    for k,v in r.items():
-        print(k, len(v))
\ No newline at end of file
+    idxData.create(begin_date=19890101)
+    datas = idxData.acquire_datas_by_date(20050101)
+    print(datas)
